#define in1 9 // defining the input pins for the motors
#define in2 6
#define in3 3
#define in4 5
#define r_led 10
#define g_led 11
#define b_led 13

char in;// defining the input variable for the bluetooth communication
char prev_in;

class motor // creating a class for creating a motor object and controlling it using functions
{
  public:

    motor(int in_1, int in_2);
    int _in1;
    int _in2;
    int val;
    void fwd(int val);
    void bck(int val);
    void off();
};

motor::motor (int in_1, int in_2) // attaches the pins in_1 and in_2 to the motor object created
{
  _in1 = in_1;
  _in2 = in_2;
  pinMode(_in1, OUTPUT); // initializes the pins to be controlled by the motor
  pinMode(_in2, OUTPUT);
}

void motor::fwd(int val) // function to mke the motor spin in one direction; val is the specification of how much voltage should be applied to the motor
{
  analogWrite(_in1, map(val, 1, 100, 50, 255));
  digitalWrite(_in2, LOW);
}

void motor::bck(int val) // performs the same function as fwd(), except that the polarity of the pins is reversed, so the motor would spin in the reverse direction
{
  analogWrite(_in2, map(val, 1, 100, 50, 255));
  digitalWrite(_in1, LOW);
}

void motor::off() // turns the motor off
{
  digitalWrite(_in1, LOW);
  digitalWrite(_in2, LOW);
}





motor motor1(in1, in2); // instantiating objects from the motor class to create motor objects
motor motor2(in3, in4);





void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);

  pinMode(10, OUTPUT);
  pinMode(11, OUTPUT);
  pinMode(12, OUTPUT);
  pinMode(13, OUTPUT);
  digitalWrite(12, HIGH);
  digitalWrite(b_led, HIGH);

  motor1.fwd(100); // testing that the motors are connect properly
  motor2.fwd(100);
  delay(2000);
  motor1.off();
  motor2.off();
  delay(250);
  motor1.bck(100);
  motor2.bck(100);
  delay(2000);
  motor1.off();
  motor2.off();
  delay(250);


}

void loop() {
  int val = 100; // specifies that all the motors are going to run at full power

  prev_in = in;

  // the following code takes the input from the serial communication via bluetooth
  if (Serial.available() > 0)
  {
    Serial.println("R");
    in = Serial.read();
  }

  // prints the input to the serial monitor

  Serial.println("I");
  Serial.println(in);


  /*
      Based on the input, the switch case structure will decide which code to execute.
      for case "w", the motor will move forward
      for case "s", the motor will move backward
      for case "a", the motor 1 will move forward and motor 2 backward, causing the robot to turn in a direction
      for case "d", the motor 2 will move forward and motor 1 backward, causing the robot to turn in another direction
      for any other case, the motors will remain off.
      This sequence is based off of common gaming controls
  */
  switch (in)
  {
    case 'w':
      Serial.println("Forward");
      if (prev_in != in)
      {
        led_off();
        motor1.off();
        motor2.off();
        delay(500);
        for (int i = 33; i <= 100; i++)
        {
          motor1.fwd(i);
          motor2.fwd(i);
          delay(33);
        }
        digitalWrite(r_led, HIGH);
      }

      motor1.fwd(val);
      motor2.fwd(val);

      break;

    case 's':
      Serial.println("Back");
      if (prev_in != in)
      {
        led_off();
        motor1.off();
        motor2.off();
        delay(500);
        for (int i = 33; i <= 100; i++)
        {
          motor1.bck(i);
          motor2.bck(i);
          delay(33);
        }
        digitalWrite(g_led, HIGH);
      }

      motor1.bck(val);
      motor2.bck(val);
      break;

    case 'a':
      Serial.println("Left");
      if (prev_in != in)
      {
        led_off();
        motor1.off();
        motor2.off();
        delay(500);
        analogWrite(r_led, 255 / 2);
        analogWrite(g_led, 255 / 2);
      }
      motor1.fwd(val);
      motor2.bck(val);

      break;

    case 'd':
      Serial.println("Right");

      if (prev_in != in)
      {
        led_off();
        motor1.off();
        motor2.off();
        delay(500);
        analogWrite(r_led, 255 / 2);
        analogWrite(g_led, 255 / 2);
      }

      motor1.bck(val);
      motor2.fwd(val);
      break;

    default:
      motor1.off();
      motor2.off();
      delay(500);
      Serial.println("Default");
      break;

  }
}

void led_off()
{
  digitalWrite(r_led, LOW);
  digitalWrite(g_led, LOW);
}

